(function() {
  var EventEmitter, StockedItem, cleanUp, config, currentStock, getItem, removeItem, request, stockCount, xml2js;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  EventEmitter = require('events').EventEmitter;
  request = require('request');
  xml2js = require('xml2js');
  currentStock = {};
  stockCount = 0;
  config = {
    shelfLife: 60,
    maxLife: 300,
    capacity: 1000,
    ideal: 900
  };
  this.configure = function(options) {
    var k, v;
    for (k in options) {
      v = options[k];
      config[k] = v;
    }
    if (!(options.ideal && config.ideal <= (config.capacity * 0.9))) {
      config.ideal = config.capacity * 0.9;
    }
    return console.log(config);
  };
  this.fetch = function(options, callback) {
    var k, stockedItem, v, _ref, _ref2;
        if ((_ref = options.key) != null) {
      _ref;
    } else {
      options.key = options.uri;
    };
    for (k in config) {
      v = config[k];
            if ((_ref2 = options[k]) != null) {
        _ref2;
      } else {
        options[k] = v;
      };
    }
    stockedItem = getItem(options);
    if (stockedItem.hasSpoiled()) {
      stockedItem.once('stocked', __bind(function(error, results) {
        return callback(error, results);
      }, this));
    } else {
      callback(null, stockedItem.results);
    }
    if (stockedItem.hasExpired()) {
      return stockedItem.fetch(options);
    }
  };
  getItem = function(options) {
    if (!(currentStock[options.key] != null)) {
      currentStock[options.key] = new StockedItem(options);
      stockCount++;
      cleanUp();
    }
    return currentStock[options.key];
  };
  removeItem = function(key) {
    delete currentStock[key];
    return stockCount--;
  };
  cleanUp = function() {
    var expired, item, key, _i, _len, _results;
    if (stockCount > config.capacity) {
      console.log("We're over capacity " + stockCount + " / " + config.ideal + ".  Time to clean up the pantry");
      expired = [];
      for (key in currentStock) {
        item = currentStock[key];
        if (item.results != null) {
          if (item.hasSpoiled()) {
            console.log("\tSpoiled: " + key);
            removeItem(key);
          } else if (item.hasExpired()) {
            expired.push(key);
          }
        }
      }
      if (stockCount > config.capacity) {
        for (_i = 0, _len = expired.length; _i < _len; _i++) {
          key = expired[_i];
          console.log("\tExpired: " + key);
          removeItem(key);
          if (stockCount <= config.ideal) {
            break;
          }
        }
      }
      if (stockCount > config.capacity) {
        _results = [];
        for (key in currentStock) {
          item = currentStock[key];
          if (item.results) {
            console.log("\tTossed: " + key);
            removeItem(key);
            if (stockCount <= config.ideal) {
              break;
            }
          }
        }
        return _results;
      }
    }
  };
  StockedItem = (function() {
    __extends(StockedItem, EventEmitter);
    function StockedItem(options) {
      this.options = options;
      this.loading = false;
    }
    StockedItem.prototype.hasExpired = function() {
      return this.hasSpoiled() || (new Date()) > this.bestBefore;
    };
    StockedItem.prototype.hasSpoiled = function() {
      return !(this.results != null) || (new Date()) > this.spoilesOn;
    };
    StockedItem.prototype.fetch = function(options) {
      var _base, _ref;
      this.options = options;
      if (this.loading) {
        return;
      }
      this.loading = true;
            if ((_ref = (_base = this.options).headers) != null) {
        _ref;
      } else {
        _base.headers = {};
      };
      if (this.eTag) {
        this.options.headers['if-none-match'] = this.eTag;
      }
      if (this.lastModified) {
        this.options.headers['if-modified-since'] = this.lastModified;
      }
      return request(this.options, __bind(function(error, response, body) {
        var contentType, parser;
        if (error == null) {
          switch (response.statusCode) {
            case 304:
              console.log("Still Good: " + options.key);
              return this.stock(response, null);
            case 200:
              console.log("Re-stocked: " + options.key);
              contentType = response.headers["content-type"];
              if (contentType.indexOf('application/json') === 0) {
                return this.stock(response, JSON.parse(body));
              } else if (contentType.search(/[\/\+]xml/) > 0) {
                parser = new xml2js.Parser();
                parser.on('end', __bind(function(results) {
                  return this.stock(response, results);
                }, this));
                return parser.parseString(body);
              } else {
                return this.oops("Invalid Response Type (" + contentType + ")");
              }
              break;
            default:
              return this.oops("Invalid Response Code (" + response.statusCode + ")");
          }
        }
      }, this));
    };
    StockedItem.prototype.stock = function(response, results) {
      this.loading = false;
      if (this.firstPurchased != null) {
        this.lastPurchased = new Date();
      } else {
        this.firstPurchased = new Date();
        this.lastPurchased = new Date(this.firstPurchased);
      }
      this.lastUsed = new Date(this.lastPurchased);
      this.bestBefore = new Date(this.lastPurchased);
      this.bestBefore.setSeconds(this.bestBefore.getSeconds() + this.options.shelfLife);
      this.spoilesOn = new Date(this.lastPurchased);
      this.spoilesOn.setSeconds(this.spoilesOn.getSeconds() + this.options.maxLife);
      this.eTag = response.headers['etag'];
      this.lastModified = response.headers['last-modified'];
      if (results != null) {
        this.results = results;
      }
      return this.emit('stocked', null, this.results);
    };
    StockedItem.prototype.oops = function(error) {
      this.loading = false;
      return this.emit('stocked', error);
    };
    return StockedItem;
  })();
}).call(this);
